<?php

namespace FiloBlu\Eva\Block\Widget;

/**
 * Class LinkWidget
 * @package FiloBlu\Eva\Block\Widget
 */
class LinkWidget extends \Magento\Framework\View\Element\Template implements \Magento\Widget\Block\BlockInterface
{

    /**
     * @var \Magento\Catalog\Model\CategoryRepository
     */
    protected $_categoryRepository;
    /**
     * @var \Magento\Cms\Model\Template\FilterProvider
     */
    protected $_filterProvider;
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product
     */
    protected $_resourceModelProduct;
    /**
     * @var \Magento\Catalog\Model\ProductRepository
     */
    protected $_productRepository;
    /**
     * @var \Magento\Cms\Helper\Page
     */
    protected $_cmsPageHelper;
    /**
     * @var \Magento\Cms\Model\PageFactory
     */
    protected $_pageFactory;
    /**
     * @var string
     */
    protected $_template = "widget/link-widget.phtml";

    /**
     * LinkWidget constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Catalog\Model\CategoryRepository $categoryRepository
     * @param \Magento\Cms\Model\Template\FilterProvider $filterProvider
     * @param \Magento\Catalog\Model\ResourceModel\Product $resourceModelProduct
     * @param \Magento\Catalog\Model\ProductRepository $productRepository
     * @param \Magento\Cms\Helper\Page $cmsPageHelper
     * @param \Magento\Cms\Model\PageFactory $pageFactory
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Catalog\Model\CategoryRepository $categoryRepository,
        \Magento\Cms\Model\Template\FilterProvider $filterProvider,
        \Magento\Catalog\Model\ResourceModel\Product $resourceModelProduct,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Cms\Helper\Page $cmsPageHelper,
        \Magento\Cms\Model\PageFactory $pageFactory
    ) {
        parent::__construct($context, $data = []);
        $this->_categoryRepository = $categoryRepository;
        $this->_filterProvider = $filterProvider;
        $this->_resourceModelProduct = $resourceModelProduct;
        $this->_productRepository = $productRepository;
        $this->_cmsPageHelper = $cmsPageHelper;
        $this->_pageFactory = $pageFactory;
    }

    /**
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getWidgetUrl()
    {
        $_url = null;
        $linkType = $this->getLinkType();
        $customUri = $this->getLinkCustom();
        $categoryId = str_replace('category/', '', (string)$this->getLinkCategory());
        $productId = str_replace('product/', '', (string)$this->getLinkProduct());
        $pageId = $this->getLinkCmspage();

        switch ($linkType) {
            case 'default':
            case 1:
                $_url = '';
                break;

            case 2:
                $_url = $this->getCategoryUrlById($categoryId);
                break;

            case 3:
                $_url = $this->getProductUrlById($productId);
                break;

            case 4:
                $_url = $this->getCmsPageUrl($pageId);
                break;

            case 5:
                $_url = $this->getFilteredContent($customUri);
                break;
        }

        return trim($_url);
    }

    /**
     * @param $categoryId
     * @return |null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCategoryUrlById($categoryId)
    {
        return ($categoryId) ? $this->_categoryRepository->get($categoryId)->getUrl() : null;
    }

    /**
     * @param $productId
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductUrlById($productId)
    {
        $product = $this->_productRepository->getById($productId);
        return $product->getUrlModel()->getUrl($product);
    }

    /**
     * @param $pageId
     * @return string
     */
    public function getCmsPageUrl($pageId)
    {
        return $this->_cmsPageHelper->getPageUrl($pageId);
    }

    /**
     * @param $string
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getFilteredContent($string)
    {
        return $this->_filterProvider->getBlockFilter()->setStoreId($this->_storeManager->getStore()->getId())->filter(
            $string
        );
    }

    /**
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getLinkText()
    {
        $_text = null;
        $linkType = $this->getLinkType();
        $customText = $this->getCustomLinkText();
        $categoryId = str_replace('category/', '', (string)$this->getLinkCategory());
        $productId = str_replace('product/', '', (string)$this->getLinkProduct());
        $pageId = $this->getLinkCmspage();

        if ($customText) {
            $_text = $customText;
        } else {
            switch ($linkType) {
                case 'default':
                case 1:
                    $_text = '';
                    break;

                case 2:
                    $_text = $this->getCategoryTitle($categoryId);
                    break;

                case 3:
                    $_text = $this->getProductName($productId);
                    break;

                case 4:
                    $_text = $this->getCmsPageTitle($pageId);
                    break;
            }
        }


        return trim($_text);
    }

    /**
     * @param $categoryId
     * @return string|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCategoryTitle($categoryId)
    {
        return ($categoryId) ? $this->_categoryRepository->get($categoryId)->getName() : null;
    }

    /**
     * @param $productId
     * @return string|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductName($productId)
    {
        $product = $this->_productRepository->getById($productId);
        return $product->getName();
    }

    /**
     * @param $pageId
     * @return string
     */
    public function getCmsPageTitle($pageId)
    {
        return $this->_pageFactory->create()->load($pageId)->getTitle();
    }

}